package com.diagbot.aggregate;

import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleBaseInitDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleConditionInitDTO;
import com.diagbot.dto.RuleDTO;
import com.diagbot.dto.RuleInitDTO;
import com.diagbot.enums.RedisEnum;
import com.diagbot.facade.KlRuleFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/5/31 17:14
 */
@Component
@Slf4j
public class RuleAggregate {
    @Autowired
    private KlRuleFacade klRuleFacade;

    @DataProvider("getAllRules")
    public Map<String, RuleDTO> getAllRules(
            @DataConsumer("getRuleInit") Map<String, List<RuleInitDTO>> ruleInits,
            @DataConsumer("getRuleConditionInit") Map<Integer, Map<String, List<RuleConditionInitDTO>>> ruleConditionInits,
            @DataConsumer("getRuleBaseInit") Map<Integer, RuleBaseInitDTO> ruleBaseInits) {
        Map<String, RuleDTO> res = new HashMap();
        for (String ruleKey : ruleInits.keySet()) {
            RuleInitDTO ruleInitZeroDTO = ruleInits.get(ruleKey).get(0);
            RuleDTO ruleDTO = new RuleDTO();
            ruleDTO.setLibName(ruleInitZeroDTO.getLibName());
            ruleDTO.setLibType(ruleInitZeroDTO.getLibType());
            ruleDTO.setRuleType(ruleInitZeroDTO.getRuleType());
            for (RuleInitDTO ruleInitDTO : ruleInits.get(ruleKey)) {
                RuleConditionDTO ruleConditionDTO = new RuleConditionDTO();
                ruleConditionDTO.setHasSubCond(ruleInitDTO.getHasSubCond());
                ruleConditionDTO.setMsg(ruleInitDTO.getMsg());
                if (ruleInitDTO.getHasSubCond().equals(1)) {
                    Map<String, List<RuleConditionInitDTO>> map = ruleConditionInits.get(ruleInitDTO.getRuleId());
                    for (String key : map.keySet()) {
                        ruleConditionDTO.setRuleGroup(key);
                        for (RuleConditionInitDTO ruleConditionInitDTO : map.get(key)) {
                            RuleBaseDTO ruleBaseDTO = new RuleBaseDTO();
                            BeanUtil.copyProperties(ruleBaseInits.get(ruleConditionInitDTO.getRuleBaseId()), ruleBaseDTO);
                            ruleConditionDTO.getRuleBaseDTOList().add(ruleBaseDTO);
                        }
                    }
                } else {
                    //无条件
                    ruleConditionDTO.setRuleGroup(ruleInitDTO.getRuleId() + "-1");
                    ruleConditionDTO.getRuleBaseDTOList().add(new RuleBaseDTO());
                }
                ruleDTO.getRuleConditionDTOList().add(ruleConditionDTO);
            }
            res.put(RedisEnum.allRule.getName() + ruleDTO.getLibType() + "_" + ruleDTO.getLibName() + "_" + ruleDTO.getRuleType(), ruleDTO);
        }
        return res;
    }

    @DataProvider("getRuleInit")
    public Map<String, List<RuleInitDTO>> getRuleInit() {
        List<RuleInitDTO> list = klRuleFacade.getRuleInitDTO();
        return EntityUtil.makeEntityListMap(list, "conceptGroup");
    }

    @DataProvider("getRuleConditionInit")
    public Map<Integer, Map<String, List<RuleConditionInitDTO>>> getRuleConditionInit() {
        List<RuleConditionInitDTO> list = klRuleFacade.getRuleConditionInitDTO();
        Map<Integer, List<RuleConditionInitDTO>> map = EntityUtil.makeEntityListMap(list, "ruleId");
        Map<Integer, Map<String, List<RuleConditionInitDTO>>> res = new HashMap<>();
        for (Integer key : map.keySet()) {
            res.put(key, EntityUtil.makeEntityListMap(map.get(key), "ruleGroup"));
        }
        return res;
    }


    @DataProvider("getRuleBaseInit")
    public Map<Integer, RuleBaseInitDTO> getRuleBaseInit() {
        List<RuleBaseInitDTO> list = klRuleFacade.getRuleBaseInitDTO();
        return EntityUtil.makeEntityMap(list, "ruleBaseId");
    }
}
